/**
 * FLL - Level 3
 *
 * Project: FSS Write
 * API Version: 0.7
 * Licenses: lgpl-2.1-or-later
 *
 * Provides the common data structures.
 *
 * This is auto-included and should not need to be explicitly included.
 */
#ifndef _fss_write_common_define_h
#define _fss_write_common_define_h

#ifdef __cplusplus
extern "C" {
#endif

/**
 * The program allocation defines.
 *
 * fss_write_allocation_*_d:
 *   - console: An allocation step used for small buffers specifically for console parameter.
 *   - large:   An allocation step used for buffers that are anticipated to have large buffers.
 *   - pipe:    A buffer size used for processing piped data.
 *   - small:   An allocation step used for buffers that are anticipated to have small buffers.
 */
#ifndef _di_fss_write_allocation_d_
  #define fss_write_allocation_console_d 4
  #define fss_write_allocation_large_d   2048
  #define fss_write_allocation_pipe_d    16384
  #define fss_write_allocation_small_d   128
#endif // _di_fss_write_allocation_d_

/**
 * Flags passed to the main function or program.
 *
 * fss_write_main_flag_*_d:
 *   - none:                   No flags set.
 *   - content:                The Content being written is specified.
 *   - content_end:            The Content end characters are to be printed.
 *   - content_multiple:       Designate that multiple Content is allowed for an Object for this standard rather than a single Content per Object.
 *   - content_next:           The Content next characters are to be printed.
 *   - copyright:              Print copyright.
 *   - file_to:                Using a specified destination file.
 *   - help:                   Print help.
 *   - ignore:                 Ignore a given range within a Content (specify flag before setting loading to designate ignores is supported by standard).
 *   - object:                 The Object being written is specified.
 *   - object_open:            The Object open characters are to be printed.
 *   - partial:                Do not write end of Object/Content character.
 *   - pipe:                   Use the input pipe.
 *   - trim:                   Trim Object names.
 *   - version:                Print version.
 *   - version_copyright_help: A helper flag representing version, copyright, and help flag bits being set.
 */
#ifndef _di_fss_write_main_flag_d_
  #define fss_write_main_flag_none_d                   0x0
  #define fss_write_main_flag_content_d                0x1
  #define fss_write_main_flag_content_end_d            0x2
  #define fss_write_main_flag_content_multiple_d       0x4
  #define fss_write_main_flag_content_next_d           0x8
  #define fss_write_main_flag_copyright_d              0x10
  #define fss_write_main_flag_file_to_d                0x20
  #define fss_write_main_flag_help_d                   0x40
  #define fss_write_main_flag_ignore_d                 0x80
  #define fss_write_main_flag_object_d                 0x100
  #define fss_write_main_flag_object_open_d            0x200
  #define fss_write_main_flag_partial_d                0x400
  #define fss_write_main_flag_pipe_d                   0x800
  #define fss_write_main_flag_prepend_d                0x1000
  #define fss_write_main_flag_trim_d                   0x2000
  #define fss_write_main_flag_version_d                0x4000
  #define fss_write_main_flag_version_copyright_help_d 0x4050
#endif // _di_fss_write_main_flag_d_

/**
 * Flags for fine-tuned print control.
 *
 * fss_write_print_flag_*_d:
 *   - none:    No flags set.
 *   - debug:   Stream is for debug printing.
 *   - error:   Stream is for error printing.
 *   - in:      Stream is a source file.
 *   - message: Stream is for message printing.
 *   - out:     Stream is a destination file.
 *   - warning: Stream is for warning printing.
 */
#ifndef _di_fss_write_print_flag_d_
  #define fss_write_print_flag_none_d    0x0
  #define fss_write_print_flag_debug_d   0x1
  #define fss_write_print_flag_error_d   0x2
  #define fss_write_print_flag_file_d    0x4
  #define fss_write_print_flag_in_d      0x8
  #define fss_write_print_flag_out_d     0x10
  #define fss_write_print_flag_message_d 0x20
  #define fss_write_print_flag_warning_d 0x40
#endif // _di_fss_write_print_flag_d_

/**
 * The program signal defines.
 *
 * fss_write_signal_*_d:
 *   - check:          Number of iterations before performing signal check in non-threaded signal handling.
 *   - check_failsafe: When using threads, how many consecutive failures to check signal before aborting (as a recursion failsafe).
 *   - check_tiny:     The tiny check.
 *   - check_short:    The short signal check.
 */
#ifndef _di_fss_write_signal_d_
  #define fss_write_signal_check_d          500000
  #define fss_write_signal_check_failsafe_d 20000
  #define fss_write_signal_check_tiny_d     4
  #define fss_write_signal_check_short_d    16
#endif // _di_fss_write_signal_d_

#ifdef __cplusplus
} // extern "C"
#endif

#endif // _fss_write_common_define_h
